/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// interceptaction.cpp Tue Jul 31 2001
// Description: attempt to make interceptball work without breaking defense
//////////////////////////////////////////////////////////////////////

#include "interceptaction.h"
#include "movecommand.h"
#include "dashcommand.h"
#include "turncommand.h"

// this bool keeps the player from turning to run twice
// in a row.  Replacing the second turn with a dash,
// ensures that the player is always moving and prevents
//  an escalation into chaotic behavior.
bool InterceptAction::fAlreadyTurned = false;

InterceptAction::InterceptAction(WorldModel& _wmWorld) : Action(), wmWorld(_wmWorld)
{
	// A kickaction takes one 100ms timestep, so iDuration is set to 1.
	// in the future this value may changed based on the worldmodel
	// and the server commands (may need to be dynamic)
	// TODO: perhaps #define somewhere
	setDuration(1);
	strcpy(pcCommand, "InterceptAction");
}

InterceptAction::InterceptAction(WorldModel& _wmWorld, const Angle& _aDirection, double _dIntensity) : Action(), wmWorld(_wmWorld)
{
	setDuration(1);
	setDirection(_aDirection);
	setIntensity(_dIntensity);
	strcpy(pcCommand, "InterceptAction");	
}

InterceptAction::~InterceptAction()
{
}

Action* InterceptAction::copy() const
{
	return new InterceptAction(*this);
}

ServerCommand* InterceptAction::translate() const
{
	// WARNING: creating a new servercommand, must remember to delete!!!
	ServerCommand* pscCmd;
	Angle aRunAngle = getDirection();	

	// scale the window where a dash is acceptable based on the power of the dash
	// NOTE: the desired direction has a wide deviation if your moving
	//			 slowly and a small deviation if your moving quickly	
//	Angle aTurnWindow = (1 - getIntensity() / MAX_DASH_POWER) * MAX_TURN_WINDOW;
//	if (aTurnWindow < MIN_TURN_WINDOW)
//	{
//		aTurnWindow = MIN_TURN_WINDOW;
//	}
	
//	Angle aTurnWindow = DEG_TO_RAD(10);
//	InterceptAction::fAlreadyTurned = (iBallAge % 2 == 1); // because we want to run on this condition
	
	// if the players current body direction is the desired direction or if we already turned last time

	ClientWorldPlayer* pcwp = wmWorld.modifyClientPlayer();
//	if (InterceptAction::fAlreadyTurned || fabs(aRunAngle) < aTurnWindow)
	if (fabs(aRunAngle) == 0.0)// < aTurnWindow)	
	{
		// create a dash server command
		pscCmd = new DashCommand(getIntensity());
		InterceptAction::fAlreadyTurned = false;
		strcpy(pcCommand, "dash");
		
		// UPDATE WORLDMODEL
//		Vector2d vAccel;
//		vAccel.setRadAng(pcwp->getEffort() * DASH_POWER_RATE * getIntensity(), pcwp->getBodyDir() );
//        pcwp->setVelocity(pcwp->getVelocity() * PLAYER_DECAY + vAccel);		
//        pcwp->setAbsPosition(pcwp->getAbsPosition() + pcwp->getVelocity());
	}
	else
	{
		// otherwise turn to the proper direction.
		pscCmd = new TurnCommand(aRunAngle);
		InterceptAction::fAlreadyTurned = true;
		strcpy(pcCommand, "turn");
		
		// UPDATE WORLDMODEL

		// Turn angle
		
		Angle aFaceDir = pcwp->getFaceDir();
		Angle aActualTurn = aRunAngle/(1.0 + INERTIA_MOMENT * pcwp->getVelocity().getLength() );
		
		Angle aSum = aFaceDir + aActualTurn;
		pcwp->setFaceDir(aSum);
        ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();

		// velocity decays
//        pcwp->setVelocity(pcwp->getVelocity() * PLAYER_DECAY);		
	}
	
	return pscCmd;
}

void InterceptAction::setDirection(const Angle& _aDirection)
{
  aDirection = _aDirection;
}

void InterceptAction::setIntensity(double _dIntensity)
{
	dIntensity = _dIntensity;
}

Angle InterceptAction::getDirection() const
{
	return aDirection;
}

double InterceptAction::getIntensity() const
{
	return dIntensity;
}

ostream& InterceptAction::ostream_insert(ostream& _out) const
{
    _out << "InterceptAction (" << getIntensity() << ", " << getDirection() << ")" << endl;

    return _out;
}
ofstream& InterceptAction::ofstream_insert(ofstream& _fout) const
{

    _fout << "<interceptaction intensity=\"" << dIntensity
          << "\" direction=\"" << RAD_TO_DEG(aDirection) << "\"/>" << endl;

    return _fout;
}

bool InterceptAction::operator==(const InterceptAction& rhs) const
{
    return (getDirection() == rhs.getDirection() &&
            getIntensity() == rhs.getIntensity());
}
