/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// runaction.cpp Fri Jun 29 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "runaction.h"
#include "movecommand.h"
#include "dashcommand.h"
#include "turncommand.h"

// this bool keeps the player from turning to run twice
// in a row.  Replacing the second turn with a dash,
// ensures that the player is always moving and prevents
//  an escalation into chaotic behavior.
bool RunAction::fAlreadyTurned = false;

RunAction::RunAction(): Action()
{
	// A kickaction takes one 100ms timestep, so iDuration is set to 1.
	// in the future this value may changed based on the worldmodel
	// and the server commands (may need to be dynamic)
	// TODO: perhaps #define somewhere
	setDuration(1);
}

RunAction::RunAction(const Angle& _aDirection, double _dIntensity) : Action()
{
	setDuration(1);
	setDirection(_aDirection);
	setIntensity(_dIntensity);
}

RunAction::~RunAction()
{
}

Action* RunAction::copy() const
{
	return new RunAction(*this);
}

ServerCommand* RunAction::translate() const
{
	// WARNING: creating a new servercommand, must remember to delete!!!
	ServerCommand* pscCmd;
	Angle aRunAngle = getDirection();	

	// scale the window where a dash is acceptable based on the power of the dash
	// NOTE: the desired direction has a wide deviation if your moving
	//			 slowly and a small deviation if your moving quickly	
	Angle aTurnWindow = (1 - getIntensity() / MAX_DASH_POWER) * MAX_TURN_WINDOW;
	if (aTurnWindow < MIN_TURN_WINDOW)
	{
		aTurnWindow = MIN_TURN_WINDOW;
	}
	
		// if the players current body direction is the desired direction or if we already turned last time
	if (RunAction::fAlreadyTurned || fabs(aRunAngle) < aTurnWindow)
//	if (fabs(aRunAngle) < aTurnWindow)	
	{
		// create a dash server command
		pscCmd = new DashCommand(getIntensity());
		RunAction::fAlreadyTurned = false;
	}
	else
	{
		// otherwise turn to the proper direction.
		pscCmd = new TurnCommand(aRunAngle);
		RunAction::fAlreadyTurned = true;
	}
	
	return pscCmd;
}

void RunAction::setDirection(const Angle& _aDirection)
{
  aDirection = _aDirection;
}

void RunAction::setIntensity(double _dIntensity)
{
	dIntensity = _dIntensity;
}

Angle RunAction::getDirection() const
{
	return aDirection;
}

double RunAction::getIntensity() const
{
	return dIntensity;
}

ostream& RunAction::ostream_insert(ostream& _out) const
{
    _out << "RunAction (" << getIntensity() << ", " << getDirection() << ")" << endl;

    return _out;
}
ofstream& RunAction::ofstream_insert(ofstream& _fout) const
{
    // TODO: Implement XML
    _fout << "<runaction intensity=\"" << dIntensity << "\" direction=\"" << RAD_TO_DEG(aDirection) << "\"/>" << endl;

    return _fout;
}
