////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// main.cpp Thu Jun  7 16:46:53 EDT 2001
// Description: This file will serve as a test harness
//////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <unistd.h>
#include "constants.h"
#include "mastercontrol.h"
#include "coachcontrol.h"
#include "debug.h"
#include <mcheck.h>

int main(int argc, char *argv[])
{
  // using mcheck to look for memory leaks, etc.
  // TODO: add mcheck to the configure script


  if (mcheck(NULL) == -1) cout << "Too late for mcheck!!" << endl;


  // definately keep
  char   *pcTeamname= "wwfc";
  char   pcHost[256] = "localhost";
  bool   fLogging = false;
  bool   fIsGoalie = false;
  bool   fIsCoach = false;
  int    iPort = 6000; // DEFAULT_PORT
  int    iReconnectNumber = -1;

  // MRB stuff, how should we deal with it?

  double mapWinMag = 0.0; // works with replayer... how hard is this to implement?
  char   *configfile= "agent.conf";

  char   c;
  while((c = getopt( argc, argv, "cgh:lm:r:p:t:f:" ) ) != EOF ) { // the colons denote strings instead of flags
    switch(c) {
    case 'c':
        fIsCoach = true; // for now, we ASSUME ONLINE COACH (bad thing to do)
        if(iPort == 6000) iPort = 6002; // Unless command line changed it
        iReconnectNumber = 0; // for Coach (used in Parser I think)
        break;
    case 'g':
      fIsGoalie = true;
      break;
    case 'h':
      // delete (pcHost);
      strcpy(pcHost, optarg);
      break;
    case 'l':
      fLogging = true;
      break;
    case 'm':
      mapWinMag = atof( optarg );
      break;
    case 'r':
      iReconnectNumber = atoi(optarg);
      if ((iReconnectNumber < 1) || (iReconnectNumber > 11))
    	goto arg_err;
      break;
    case 'p':
      iPort = atoi(optarg);
      break;
    case 't':
      pcTeamname = optarg;
      break;
    case 'f':
      configfile = optarg;
      break;
    case '?':
    arg_err:		
      cerr << "Usage: " << argv[0] << "[-h HOSTNAME]"
              " [-l] [-m MAGNIFY] [-r num] [-p PORT] [-t TEAMNAME]"
              "[-f FILENAME]" << endl
      	     << "  -g           agent is goalie" << endl
        	 << "  -h HOSTNAME  connects to HOSTNAME" << endl
        	 << "  -l           enable server message logging (to \"world*.log\")" << endl
        	 << "  -m MAGNIFY   enable mapwindow with size magnification factor MAGNIFY" << endl
        	 << "  -r num       reconnect with that number" << endl
        	 << "  -p PORT      port number" << endl
      	     << "  -t TEAMNAME  sets the player's team name" << endl
        	 << "  -f FILENAME  alternative configuration file for tactics (default agent.conf)" << endl;
      exit(1);
    }
  }

  cerr << "UVA RoboCup player" << endl;

 	loadConfig(configfile);
 	
  if(!fIsCoach)
  {
    cout << "Starting player for team " << pcTeamname << endl;	
    MasterControl mcHammer(pcTeamname, pcHost, iPort, fLogging, iReconnectNumber, fIsGoalie);
		mcHammer.play();
  }
  else
  {
    cout << "Starting Online Coach for team " << pcTeamname << endl;
    CoachControl ccTwoFifty(pcTeamname, pcHost, iPort, iReconnectNumber);
    ccTwoFifty.coach();
  }

  cout << "Exiting gracefully." << endl;
  return 0;
}


