/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// backlinebehavior.cpp Fri Jul 13 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "backlinebehavior.h"

BackLineBehavior::BackLineBehavior(const WorldModel &_wmWorld): perBackLine(_wmWorld)
{
}

BackLineBehavior::~BackLineBehavior()
{
}

bool BackLineBehavior::isUseful()
{
	if (!perBackLine.isActiveMode())
		return false;

	return  ( (!perBackLine.amIClosestToBall()) && (perBackLine.isBehindBall()) && (perBackLine.getBallAge() < 15) );
}

void BackLineBehavior::evaluate(RecommendationList& recList)
{
  //create the Recommendation to be passed back
  BackLineRecommendation blrBackLineRec;
  blrBackLineRec.szBehaviorName = "backline";
  
  double dBallDistMax;
  if (perBackLine.getPossession() == PS_OURS)
    dBallDistMax = OFFENSIVE_BALL_DIST;
  else
    dBallDistMax = DEFENSIVE_BALL_DIST;
  
  double dDistToBall =  perBackLine.distToBallX();
  /*  if (dDistToBall > dBallDistMax)
      dDistToBall = dBallDistMax;
  */	
  int iGrade = 0;
  if (dDistToBall > 50.0)
    iGrade = 70;
  else if ( (dDistToBall <= 50.0) && (dDistToBall > 40.0) )
    iGrade = 60;
  else if ( (dDistToBall <= 40.0) && (dDistToBall > 30.0) )
    iGrade = 50;
  else if ( (dDistToBall <= 30.0) && (dDistToBall > 15.0) )
    iGrade = 40;
  
  
  //int iGrade = (dDistToBall / dBallDistMax) * 100;
  
  blrBackLineRec.setGrade(iGrade);
  
  //set the action of the behind ball move and return recommendation
  Angle aRunAngle = (2.0 * PI) - perBackLine.getBodyDir();
  
  RunAction raTemp;
  
  if(perBackLine.slowRun())
    raTemp.setIntensity(15);
  else
    raTemp.setIntensity(50);
  
  raTemp.setDirection(aRunAngle);
  blrBackLineRec.addAction ( raTemp );
  raTemp.setDirection(0);
  blrBackLineRec.addAction( raTemp );
  
  
  recList.insert(blrBackLineRec);
}
