/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// coverperception.cpp Thu Jul 26 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "coverperception.h"

CoverPerception::CoverPerception(const WorldModel &_wmWorld) : Perception(_wmWorld)
{

}

CoverPerception::~CoverPerception()
{

}


WorldPlayer* CoverPerception::getCoverPlayer() const
{
  int iArray = 0;
  for (int h = 1; h <= MAX_PLAYER; ++h)
    {
      if (( wmWorld.getOtherPlayer(h).isYoung(4) )
	  && (inDefensiveThird(wmWorld.getOtherPlayer(h))) )
	++iArray;
    }
  
  if (iArray == 0)
    return NULL;
  
  // create array for distances to ball
  WorldPlayer wpOpponentArray[iArray];
  WorldPlayer temp;
  
  //sets up teammate array
  int iArrayCount = 0;
  for (int k = 1; k <= MAX_PLAYER; ++k)
    {
      if (( wmWorld.getOtherPlayer(k).isYoung(4) )
	  && (inDefensiveThird(wmWorld.getOtherPlayer(k))) )
	{
	  wpOpponentArray[iArrayCount] = wmWorld.getOtherPlayer(k);
	  ++iArrayCount;
	}
    }
  
  // get distance from the ball and resort the distance array
  for (int i = 0; i < iArray; ++i)
    {
      for (int j = 0; j < i; ++j)
	{
	  if ( dist(wpOpponentArray[j], wmWorld.getOurGoal())  > dist (wpOpponentArray[i], wmWorld.getOurGoal()) )
	    {
	      temp = wpOpponentArray [i];
	      wpOpponentArray [i] = wpOpponentArray [j];
	      wpOpponentArray [j] = temp;
	    }
	}
    }
  
  double dDistToOpponent;
  
  WorldPlayer* CoverPlayer = NULL;
  
  for (int g = 0; g < iArray; ++g)
    {
      dDistToOpponent = dist(wpOpponentArray [g], wmWorld.getClientPlayer());
      if ( isSmallestTeamDist(wpOpponentArray [g], dDistToOpponent))
	{
	  CoverPlayer = &wpOpponentArray[g];
	  break;
	}
    }
  
  
  return CoverPlayer;
  
}

bool CoverPerception::inDefensiveThird(WorldObject _woObj) const
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is less than -1/6 Length because less than that
	//is the defensive third of the field
	if ( vAbsPos.getX() < -(FIELD_LENGTH / 6.0) )
		return true;
	else return false;
}

bool CoverPerception::tooFarUp () const
{
  //gets the absolute position of the WMO
  ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();

  return (cwpClientPlayer.getAbsPosition().getX () > -5.0);
}

bool CoverPerception::isSmallestTeamDist(const WorldObject& _woObj, double _dDist) const
{
	//ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	int iClientNum = wmWorld.getClientPlayer().getNumber();

	for (int h = 1; h <= MAX_PLAYER; ++h)
	{
		if (h != iClientNum)
		{
			if ( dist ( _woObj, wmWorld.getOurPlayer(h)) < _dDist )
				return false;
		}
	}
	
	return true;
}

Angle CoverPerception::getBodyDir() const
{
 	return ( wmWorld.getClientPlayer().getBodyDir() );
}

bool CoverPerception::onGoalSide(const WorldObject& _woObj)
{
	if (wmWorld.getClientPlayer().getAbsPosition().getX() < _woObj.getAbsPosition().getX())
		return true;
	else
		return false;
}

WorldBall CoverPerception::getBall()
{
	return wmWorld.getWorldBall();
}

bool CoverPerception::isActiveMode() const
{
    play_mode pmTemp = wmWorld.getPlayMode();
    if( (pmTemp == PLAY_MODE_PLAY_ON) ||
        (pmTemp == PLAY_MODE_DROP_BALL)
      ) return true;

    return false;
}
