//////////////////////////////////////////////////////////////////////
// Mainz Rolling Brains RoboCup agents developed for the Simulator 
// League of the RoboCup 2000 competition
//
// Copyright (C) 1998, 1999, 2000 Team Members 
// of the Mainz Rolling Brains RoboCup Team 
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
//   $Id: grade.h,v 1.4 2001/07/05 14:05:54 kmb3s Exp $	
//   $Manager: cm$
//
//   grade.h
//
////////////////////////////////////////////////////////////////////////

#ifndef _GRADE_H_
#define _GRADE_H_

#include <iostream.h>
#include <math.h>

// This comment is obsolete:
//
// The class "Grade" stores and manipulates valuations evaluations modelled
// after (German) school grades. It is e.g. used to evaluate agent actions.
// "Grade 0" means absoluteness, "Grade 0" impossibility. The different
// grades are not displayed as numbers but with words:
//              0 <-> ABSOLUTE
//              1 <-> PERFECT
//              2 <-> GOOD
//              3 <-> AVERAGE
//              4 <-> POOR
//              5 <-> BAD
//              6 <-> IMPOSSIBLE
// CAUTION: IMPOSSIBLE is no bad grade but means impossibility.
// The integer value (on the left side) is private, the different grades have
// their own methods. (This should prevent "adding" of grades or other
// weird things.)
// Values not in {0,...,6} leave the grade undefined. Comparison with
// undefined grades will always return false.
// CAUTION: There are no OPERATORS for comparison. (Not good in a system where
// higher numbers mean worse grades.)

// WARNING: declaring DBL_EPSILON because cannot be found anywhere

#ifndef DBL_EPSILON
#define DBL_EPSILON 0.1
#endif

class Grade
{
private:
  double value;
  
  // comparison operator made private (so that you don't use it)
  static bool doubleCloseEnough (double f1, double f2)
    { return (fabs (f1 - f2) < (DBL_EPSILON * 2.0)); }

  static bool doubleIsBigger (double f1, double f2)
    { return ((f1 - f2) > DBL_EPSILON); }
      
public:
  Grade() : value( -1 ) {}
  Grade(float p_v) : value (p_v) {}

  // methods for manipulation

  void set (double p_v) { value = p_v; }

  void set_ABSOLUTE()
  { value = 100.0; }

  void set_PERFECT()
  { value = 90.0; }

  void set_GOOD()
  { value = 70.0; }

  void set_AVERAGE()
  { value = 50.0; }

  void set_POOR()
  { value = 30.0; }

  void set_BAD()
  { value = 10.0; }

  void set_IMPOSSIBLE()
  { value = 0.0; }

  // upgrade (CAUTION: PERFECT can not be upgraded to ABSOLUTE, IMPOSSIBLE
  // can not be upgraded to BAD)
  bool upgrade()
  { if (value > 0.0 && value < 90.0 ) { value += 10.0; return true; } return false; }

  // downgrade (CAUTION: ABSOLUTE can not be downgraded to PERFECT, BAD
  // can not be downgraded to IMPOSSIBLE)
  bool downgrade()
  { if (value > 10.0 && value < 99.9 ) { value -= 10.0; return true; } return false; }
    
  // methods for query and comparison
  
  bool is_defined() const
  { return value >= 0.0 && value <= 100.0 ? true : false; }
  
  bool is_ABSOLUTE() const
    { return doubleCloseEnough (value, 100.0); }

  bool is_PERFECT() const
    { return doubleCloseEnough (value, 90.0); }

  bool is_GOOD() const
    { return doubleCloseEnough (value, 70.0); }
  
  bool is_AVERAGE() const
    { return doubleCloseEnough (value, 50.0); }

  bool is_POOR() const
    { return doubleCloseEnough (value, 30.0); }

  bool is_BAD() const
    { return doubleCloseEnough (value, 10.0); }

  bool is_IMPOSSIBLE() const
    { return doubleCloseEnough (value, 0.0); }
  
  bool is_like( const Grade& g ) const
    { return is_defined() && g.is_defined() && doubleCloseEnough (value, g.value); }
  
  bool is_better_than( const Grade& g ) const
    { return is_defined() && g.is_defined() && doubleIsBigger (value, g.value); }
  
  bool is_worse_than( const Grade& g ) const
    { return is_defined() && g.is_defined() && doubleIsBigger (g.value, value); }

  double getValue () const
    { return value; }


  // TODO: use epsilon error stuff on these operators (like on ==)
	bool operator>(const Grade& g) const
		{ return (getValue() > g.getValue()); }
	
	bool operator<(const Grade& g) const
		{ return (getValue() < g.getValue())	; }
		
	bool operator>=(const Grade& g) const
		{ return (getValue() >= g.getValue()); }
	
	bool operator<=(const Grade& g) const
		{ return (getValue() <= g.getValue())	; }

  bool operator == ( const Grade& g ) const
  { return is_defined() && g.is_defined() && doubleCloseEnough (value, g.value); }
	
};

#endif



