////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// audiodata.cpp Wed Jun 13 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "audiodata.h"
#include "debug.h"

//#undef TRACE
//#define TRACE(cmd)

AudioData::AudioData()
{}

AudioData::AudioData(int _time) : SensorData(_time)
{}

AudioData::~AudioData()
{}


WorldModel& AudioData::worldmodel_insert(WorldModel &_wmWorld) const
{
    if(sender == SENDER_REFEREE)
    {
        _wmWorld.setPlayMode(mode);

        if(mode == PLAY_MODE_GOAL_US)
	  _wmWorld.setOurScore(score);

        if(mode == PLAY_MODE_GOAL_THEM)
            _wmWorld.setOtherScore(score);

        return _wmWorld;
    }

    if(sender == SENDER_DIR)
    {
        int msg_index = 0;

//        TRACE("*** received message from player " << message);
        


        char pcObject[MAX_SAY_MSG_SIZE];
        char *ObjectItr = &pcObject[0];

        char team = 0;
        int age, number;
        float x, y, xvel, yvel;
        int bodydir, headdir;

        int paren = 0;
        while(message[msg_index])
        {
            ObjectItr = &pcObject[0];
            while(message[msg_index])
            {
                if(message[msg_index] == '(')
                    paren++;
                else if(message[msg_index] == ')')
                    paren--;

                *ObjectItr = message[msg_index];

                if(paren == 0)
                    break;

                ObjectItr++;
                msg_index++;
            }

            // Parse the message
            if(sscanf(pcObject, "((p %d %c %d) %f %f %f %f %d %d)", &age, &team, &number,
                      &x, &y, &xvel, &yvel, &bodydir, &headdir) == 9)
            {
                ENSURE((number > 0) && (number <= MAX_PLAYER));
                // Test for clientplayer ... trust self, not others
                WorldPlayer wpPlayer;
                if(team == 'u')
                {
                    if(age > _wmWorld.getOurPlayer(number).getAge())
                        continue; // bail
                    if(number == _wmWorld.getClientPlayer().getNumber())
                        continue; // bail
                    wpPlayer.setTeam(TEAM_US);
                }
                else if(team == 't')
                {
                    if(age > _wmWorld.getOtherPlayer(number).getAge())
                        continue; // bail
                    wpPlayer.setTeam(TEAM_THEM);
                }
                else
                {
                    wpPlayer.setTeam(TEAM_UNKNOWN);
                    // just bail
                    continue;
                }

                wpPlayer.setAbsPosition(Vector2d(x, y));
                wpPlayer.setVelocity(Vector2d(xvel, yvel));
                wpPlayer.setNumber(number);
                wpPlayer.setBodyDir(DEG_TO_RAD(bodydir));
                wpPlayer.setHeadDir(DEG_TO_RAD(headdir));
                wpPlayer.setAge(age + 1);
                TRACE("Updating worldplayer " << wpPlayer);
                _wmWorld.updateWorldPlayer(wpPlayer);
            }
            else if(sscanf(pcObject, "((b %d) %f %f %f %f)", &age, &x, &y, &xvel, &yvel) == 5)
            {
                // update ball only if we have better info
                if(age < _wmWorld.getWorldBall().getAge())
                {

                    WorldBall wbBall;
                    wbBall.setAbsPosition(Vector2d(x, y));
                    wbBall.setVelocity(Vector2d(xvel, yvel));
                    wbBall.setAge(age + 1);
                    TRACE("Updating worldball " << wbBall);
                    _wmWorld.updateWorldBall(wbBall);

                }
            }
        }


        return _wmWorld;
    }

    if(sender == SENDER_SELF)
    {
        // TRACE("*** received message from self: " << message);

        return _wmWorld;
    }

    return _wmWorld;
}

